source("helper.R")
library("RColorBrewer")
library("ggplot2")
library(ggrepel)
library(gridExtra)
library(cowplot)
library(ggpubr)

master <- read.csv("../03_LMDI_analysis/LMDI_results_X2015_X2050.csv") %>%
  mutate(M_total = M_T2 - M_T1) %>%
  filter(region != "USA")

factor_group_mapping <- read.csv("factor_group_mapping.csv")
state_group_mapping <- read.csv("../00_data/state_group_mapping.csv") 

is_outlier <- function(x) {
  return((x < quantile(x, 0.25) - 1.5 * IQR(x) | x > quantile(x, 0.75) + 1.5 * IQR(x)) & abs(x) > 0.1)
}

base <- master %>%
  select(region, M_T1, M_T2) %>%
  mutate(code = M_T2 / M_T1) 

percentage <- master[,2:19] / master$M_T1 

percentage <- data.frame(percentage) %>%
  mutate(code = M_T2) 

# create both absolute decomposition and relative contribution
master_percentage <- base %>%
  left_join(percentage, by = "code") %>%
  select(-M_T1.x, -M_T1.y, -M_T2.x, -M_T2.y, -code) %>%
  gather(key, pct, -region) 

master_absolute <- master %>%
  select(-M_T1, -M_T2) %>%
  gather(key, value, -region)

working <- left_join(master_absolute, master_percentage, by = c("region", "key")) %>%
  left_join(factor_group_mapping, by = c("key")) %>%
  mutate(value = value/1000) %>%
  mutate(pct_label = paste0(round(100*pct, 1), "%")) %>%
  mutate(pct_label_pos = ifelse(pct >= 0, pct_label, " ")) %>%
  mutate(pct_label_neg = ifelse(pct < 0, pct_label, " "))


working_cluster <- left_join(working, state_group_mapping, by = "region") %>%
  filter(region %!in% c("USA", "HI", "DC", "AK")) %>%
  group_by(cluster, key) %>%
  mutate(outlier = ifelse(is_outlier(pct), as.character(region), as.character(NA))) %>%
  ungroup() 

write.csv(working_cluster, "boxplot_master_trend.csv", row.names = F)
  

LMDI_box_chart <- function (inputdata = working_cluster){
  x_positions <- rev(c("M_total", "M_POP", "M_GDP", "M_ENERGY", "M_ELEC_COAL", "M_INDUS_COAL", 
                       "M_INDUS_LIQ", "M_TRANS_LIQ", "M_BUILD_BIO", "M_OTHER", "M_PM_ELEC_COAL", 
                       "M_PM_INDUS_COAL", "M_PM_INDUS_LIQ", "M_PM_TRANS_LIQ", "M_PM_BUILD_BIO", 
                       "M_PM_OTHER"))
  
  x_labels <- rev(c("Net", "Population", "GDP per capta", "Energy intensity", 
                    "Fuel shares", "", "", "", "", "", "PM mortality \n intensity", 
                    "", "", "", "", ""))
  # the position of "other"
  LBL_position <- -0.58
  LBL_shift <- -0.3
  
  p <- ggplot(data = inputdata, aes(x = key, y = pct, fill = group)) +
    facet_wrap( ~ cluster, ncol = 2) +
    geom_boxplot(position = position_dodge(0.8), color = "black",
                 outlier.colour = "gray") +
    geom_text_repel(aes(label = outlier), na.rm = TRUE, nudge_x=0.06, size = 4.5, 
                    color = "darkgray", segment.color = "darkgray") +
    coord_flip() +
    geom_vline(xintercept = 15.5, colour = "black", linetype="dashed", size = 0.5) +
    # geom_vline(xintercept = 12.5, colour = "gray10", linetype="dashed", size = 0.3) +
    # geom_vline(xintercept = 6.5, colour = "gray10", linetype="dashed", size = 0.3) +
    geom_hline(yintercept = 0, colour = "black", size = 0.5) +
    xlab("") +
    ylab("Contributions") +
    scale_fill_manual(breaks = rev(c("M_total", "M_POP", "M_GDP", "M_ENERGY", "M_ELEC_COAL", "M_INDUS_COAL", 
                                     "M_INDUS_LIQ", "M_TRANS_LIQ", "M_BUILD_BIO", "M_OTHER", "M_PM_ELEC_COAL", 
                                     "M_PM_INDUS_COAL", "M_PM_INDUS_LIQ", "M_PM_TRANS_LIQ", "M_PM_BUILD_BIO", 
                                     "M_PM_OTHER")),
                      values = brewer.pal(n = 6, name = "Set1")) +
                      # values = c("red", "lightblue", "green", "orange", "pink", "purple")) +
    scale_x_discrete(limits = x_positions, labels = x_labels) + 
    scale_y_continuous(limits = c(-1, 0.76), labels = scales::percent) +
    theme_bw() +
    theme(axis.text.y   = element_text(size=16, colour = "black"),
          axis.text.x   = element_text(size=16, colour = "black"),
          axis.title.y  = element_blank(),
          axis.title.x  = element_text(size=24),
          strip.text.x = element_text(size=24),
          strip.text.y = element_text(size=28),
          strip.background = element_blank(),
          panel.background = element_blank(),
          panel.grid.major = element_blank(), 
          panel.grid.minor = element_blank(),
          panel.border = element_rect(colour = "black", fill=NA, size=1)
    ) +
    annotate(geom="text", x=6, y= LBL_position*0.86 + LBL_shift, label="Electricity coal",
             color="black", size = 5, fontface = 3) +
    annotate(geom="text", x=5, y= LBL_position*0.87 + LBL_shift, label="Industry coal ",
             color="black", size = 5, fontface = 3) +
    annotate(geom="text", x=4, y= LBL_position*0.93 + LBL_shift, label="Industry oil",
             color="black", size = 5, fontface = 3) +
    annotate(geom="text", x=3, y= LBL_position*0.65 + LBL_shift, label="Transportation liquids",
             color="black", size = 5, fontface = 3) +
    annotate(geom="text", x=2, y= LBL_position*0.78 + LBL_shift, label="Building biomass",
             color="black", size = 5, fontface = 3) +
    annotate(geom="text", x=1, y= LBL_position*1.1  + LBL_shift, label="Other",
             color="black", size = 5, fontface = 3) +
    annotate(geom="text", x=12, y= LBL_position*0.86 + LBL_shift, label="Electricity coal",
             color="black", size = 5, fontface = 3) +
    annotate(geom="text", x=11, y= LBL_position*0.87 + LBL_shift, label="Industry coal ",
             color="black", size = 5, fontface = 3) +
    annotate(geom="text", x=10, y= LBL_position*0.93 + LBL_shift, label="Industry oil",
             color="black", size = 5, fontface = 3) +
    annotate(geom="text", x=9, y= LBL_position*0.65 + LBL_shift, label="Transportation liquids",
             color="black", size = 5, fontface = 3) +
    annotate(geom="text", x=8, y= LBL_position*0.78 + LBL_shift, label="Building biomass",
             color="black", size = 5, fontface = 3) +
    annotate(geom="text", x=7, y= LBL_position*1.1      + LBL_shift, label="Other",
             color="black", size = 5, fontface = 3) +
    
    annotate(geom="text", x= c(6, 12, 14, 15, 16), y= 0.75, label="*",
             color="blue", size = 12, fontface = "bold") 

  return(p)
  
}

p <- LMDI_box_chart()
png("LMDI_two_clusters.png", width = 3600, height = 2700, res = 300)
par(cex = 2, mar = c(4, 4, 1, 1))
p
dev.off()



